@@ -1,7 +1,7 @@
 /*
  * Copyright 2003-2007 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/paxelf.c,v 1.70 2010/01/15 12:06:37 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/paxelf.c,v 1.72 2012/04/29 06:24:47 vapier Exp $
  *
  * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
@@ -675,56 +675,6 @@ char *pax_short_pf_flags(unsigned long flags)
 	return buffer;
 }
 
-unsigned long pax_pf2hf_flags(unsigned long paxflags)
-{
-	unsigned long flags = 0;
-	char *pf_flags = pax_short_pf_flags(paxflags);
-	size_t x, len = strlen(pf_flags);
-	for (x = 0; x < len; x++) {
-		switch (pf_flags[x]) {
-			case 'p':
-				flags |= HF_PAX_PAGEEXEC;
-				break;
-			case 'P':
-				flags = (flags & ~HF_PAX_PAGEEXEC) | HF_PAX_SEGMEXEC;
-				break;
-			case 'E':
-				flags |= HF_PAX_EMUTRAMP;
-				break;
-			case 'e':
-				flags = (flags & ~HF_PAX_EMUTRAMP);
-				break;
-			case 'm':
-				flags |= HF_PAX_MPROTECT;
-				break;
-			case 'M':
-				flags = (flags & ~HF_PAX_MPROTECT);
-				break;
-			case 'r':
-				flags |= HF_PAX_RANDMMAP;
-				break;
-			case 'R':
-				flags = (flags & ~HF_PAX_RANDMMAP);
-				break;
-			case 'X':
-				flags |= HF_PAX_RANDEXEC;
-				break;
-			case 'x':
-				flags = (flags & ~HF_PAX_RANDEXEC);
-				break;
-			case 's':
-				flags |= HF_PAX_SEGMEXEC;
-				break;
-			case 'S':
-				flags = (flags & ~HF_PAX_SEGMEXEC) | HF_PAX_PAGEEXEC;
-				break;
-			default:
-				break;
-		}
-	}
-	return flags;
-}
-
 char *gnu_short_stack_flags(unsigned long flags)
 {
 	static char buffer[4];
@@ -756,7 +706,6 @@ void *elf_findsecbyname(elfobj *elf, const char *name)
 	if (shstrndx >= shnum) return NULL; \
 	strtbl = &(shdr[shstrndx]); \
 	for (i = 0; i < shnum; ++i) { \
-		if (EGET(shdr[i].sh_offset) >= elf->len - EGET(ehdr->e_shentsize)) continue; \
 		offset = EGET(strtbl->sh_offset) + EGET(shdr[i].sh_name); \
 		if (offset >= (Elf ## B ## _Off)elf->len) continue; \
 		shdr_name = elf->data + offset; \
