@@ -1,13 +1,13 @@
 /*
  * Copyright 2003-2007 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.240 2012/01/23 23:48:54 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanelf.c,v 1.244 2012/04/29 06:21:36 vapier Exp $
  *
  * Copyright 2003-2007 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2004-2007 Mike Frysinger  - <vapier@gentoo.org>
  */
 
-static const char rcsid[] = "$Id: scanelf.c,v 1.240 2012/01/23 23:48:54 vapier Exp $";
+static const char rcsid[] = "$Id: scanelf.c,v 1.244 2012/04/29 06:21:36 vapier Exp $";
 const char argv0[] = "scanelf";
 
 #include "paxinc.h"
@@ -155,19 +155,35 @@ static void scanelf_file_get_symtabs(elfobj *elf, void **sym, void **str)
 	void *dynsym = elf_findsecbyname(elf, ".dynsym");
 	void *dynstr = elf_findsecbyname(elf, ".dynstr");
 
+	/*
+	 * If the sections are marked NOBITS, then they don't exist, so we just
+	 * skip them.  This let's us work sanely with splitdebug ELFs (rather
+	 * than spewing a lot of "corrupt ELF" messages later on).  In malformed
+	 * ELFs, the section might be wrongly set to NOBITS, but screw em.
+	 */
 #define GET_SYMTABS(B) \
 	if (elf->elf_class == ELFCLASS ## B) { \
-	if (symtab && dynsym) { \
-		Elf ## B ## _Shdr *esymtab = symtab; \
-		Elf ## B ## _Shdr *edynsym = dynsym; \
+	Elf ## B ## _Shdr *esymtab = symtab; \
+	Elf ## B ## _Shdr *estrtab = strtab; \
+	Elf ## B ## _Shdr *edynsym = dynsym; \
+	Elf ## B ## _Shdr *edynstr = dynstr; \
+	\
+	if (symtab && EGET(esymtab->sh_type) == SHT_NOBITS) \
+		symtab = NULL; \
+	if (dynsym && EGET(edynsym->sh_type) == SHT_NOBITS) \
+		dynsym = NULL; \
+	if (symtab && dynsym) \
 		*sym = (EGET(esymtab->sh_size) > EGET(edynsym->sh_size)) ? symtab : dynsym; \
-	} else \
+	else \
 		*sym = symtab ? symtab : dynsym; \
-	if (strtab && dynstr) { \
-		Elf ## B ## _Shdr *estrtab = strtab; \
-		Elf ## B ## _Shdr *edynstr = dynstr; \
+	\
+	if (strtab && EGET(estrtab->sh_type) == SHT_NOBITS) \
+		strtab = NULL; \
+	if (dynstr && EGET(edynstr->sh_type) == SHT_NOBITS) \
+		dynstr = NULL; \
+	if (strtab && dynstr) \
 		*str = (EGET(estrtab->sh_size) > EGET(edynstr->sh_size)) ? strtab : dynstr; \
-	} else \
+	else \
 		*str = strtab ? strtab : dynstr; \
 	}
 	GET_SYMTABS(32)
@@ -299,16 +315,12 @@ static char *scanelf_file_pax(elfobj *elf, char *found_pax)
 	SHOW_PAX(64)
 	}
 
-	if (fix_elf && setpax) {
-		/* set the chpax settings */
-		if (elf->elf_class == ELFCLASS32) {
-			if (EHDR32(elf->ehdr)->e_type == ET_DYN || EHDR32(elf->ehdr)->e_type == ET_EXEC)
-				ESET(EHDR32(elf->ehdr)->e_ident[EI_PAX],  pax_pf2hf_flags(setpax));
-		} else {
-			if (EHDR64(elf->ehdr)->e_type == ET_DYN || EHDR64(elf->ehdr)->e_type == ET_EXEC)
-				ESET(EHDR64(elf->ehdr)->e_ident[EI_PAX],  pax_pf2hf_flags(setpax));
-		}
-	}
+	/* Note: We do not support setting EI_PAX if not PT_PAX_FLAGS
+	 * was found.  This is known to break ELFs on glibc systems,
+	 * and mainline PaX has deprecated use of this for a long time.
+	 * We could support changing PT_GNU_STACK, but that doesn't
+	 * seem like it's worth the effort. #411919
+	 */
 
 	/* fall back to EI_PAX if no PT_PAX was found */
 	if (!*ret) {
@@ -1703,7 +1715,8 @@ static int scanelf_dirat(int dir_fd, const char *path)
 	if (be_verbose > 1) printf("%s: scanning dir\n", path);
 
 	subpath = stpcpy(buf, path);
-	*subpath++ = '/';
+	if (subpath[-1] != '/')
+		*subpath++ = '/';
 	pathlen = subpath - buf;
 	while ((dentry = readdir(dir))) {
 		if (!strcmp(dentry->d_name, ".") || !strcmp(dentry->d_name, ".."))
@@ -2037,16 +2050,22 @@ static void usage(int status)
 	printf("* Scan ELF binaries for stuff\n\n"
 	       "Usage: %s [options] <dir1/file1> [dir2 dirN file2 fileN ...]\n\n", argv0);
 	printf("Options: -[%s]\n", PARSE_FLAGS);
-	for (i = 0; long_opts[i].name; ++i)
-		if (long_opts[i].has_arg == no_argument)
-			printf("  -%c, --%-14s* %s\n", long_opts[i].val,
-			       long_opts[i].name, opts_help[i]);
-		else if (long_opts[i].val > '~')
-			printf("      --%-7s <arg> * %s\n",
-			       long_opts[i].name, opts_help[i]);
+	for (i = 0; long_opts[i].name; ++i) {
+		/* first output the short flag if it has one */
+		if (long_opts[i].val > '~')
+			printf("      ");
 		else
-			printf("  -%c, --%-7s <arg> * %s\n", long_opts[i].val,
-			       long_opts[i].name, opts_help[i]);
+			printf("  -%c, ", long_opts[i].val);
+
+		/* then the long flag */
+		if (long_opts[i].has_arg == no_argument)
+			printf("--%-14s", long_opts[i].name);
+		else
+			printf("--%-7s <arg> ", long_opts[i].name);
+
+		/* finally the help text */
+		printf("* %s\n", opts_help[i]);
+	}
 
 	puts("\nFor more information, see the scanelf(1) manpage");
 	exit(status);
